# -*- coding: utf-8 -*-
# (c) Jan Schween 2005 (gnuplot)
# (c) Mario Mech 2009 (python)
# (c) Maximilian Maahn 2011 (python)

from __future__ import absolute_import, division, print_function
import numpy as np

# from .due import due, Doi

import meteo_si.constants
import meteo_si.humidity
# from .temperature import *


__all__ = ["moist_rho_rh", "moist_rho_q"]


def moist_rho_rh(p, T, rh, *qm):
    """
    Compute the arithmetic circular mean, ignoring NaNs.

    Parameters
    ----------
    p :
        Pressure in Pa
    T:
        Temperature in K
    rh:
        Relative humidity in Pa/Pa
    *qm, list, optional
        mixing ratios in kg/kg of other species which contribute to the air
        mass! (ice, snow, cloud etc.)

    Returns
    -------

    float :
        Density of moist air [kg/m^3]

    Example
    -------
    moist_rho_rh(p,T,rh,q_ice,q_snow,q_rain,q_cloud,q_graupel,q_hail)

    """
    with np.errstate(divide='ignore', invalid='ignore'):
        if np.any(rh > 5):
            raise TypeError("rh must not be in %")

    q = meteo_si.humidity.rh2q(rh, T, p)

    return moist_rho_q(p, T, q, *qm)


def moist_rho_q(p, T, q, *qm):
    """
    Compute the arithmetic circular mean, ignoring NaNs.

    Parameters
    ----------
    p :
        Pressure in Pa
    T:
        Temperature in K
    q:
        specific humidity in kg/kg
    *qm, list, optional
        mixing ratios in kg/kg of other species which contribute to the air
        mass! (ice, snow, cloud etc.)

    Returns
    -------

    float :
        Density of moist air [kg/m^3]

    Example
    -------
    moist_rho_q(p,T,q,q_ice,q_snow,q_rain,q_cloud,q_graupel,q_hail)

    """
    if len(qm) > 0:
        # get rid of masked data!
        qm[qm < 0] = 0
        qm = np.sum(qm, axis=0)
    else:
        qm = 0

    moist_rho_q = p / (meteo_si.constants.Rair * T *
                       (1 + (meteo_si.constants.Rvapor /
                        meteo_si.constants.Rair-1) * q - qm))

    if np.any(moist_rho_q < 0):
        if np.any(moist_rho_q < -0.001):
            raise ValueError("calculated negative densities!")

    return moist_rho_q
